/*
 * FMDLog
 * Wideband O2 Display/Datalogging Application
 * For the TechEdge DIY-WB series of devices
 * (c) 2002, Jonathan Burchmore
 *
 * Your use of this software is governed by the GNU General Public License, v2.
 * See the file COPYING.txt, or http://www.fsf.org/copyleft/gpl.txt
 *
 * This software is provided with ABSOLUTELY NO WARRANTY.  See the GNU General
 * public license for additional information.
 */

#ifndef __fmdlog_h
#define __fmdlog_h

/*
 * Global Variables
 */

#define WBO2_ID					'WBO2'
#define WBO2_DBTYPE				'wbld'
#define WBO2_DBNAME				"WBO2_Data"
#define WBO2_CONFIG_DBTYPE		'wbcf'
#define WBO2_CONFIG_DBNAME		"WBO2_Config"

#define V_SCALE( x )			( ( ( UInt32 ) (x) * 5000 ) / 8192 )
#define V2_VWB_SCALE( x )		( ( ( UInt32 ) (x) * 5000 ) / 4096 )

#define WBO2_BITMAP_MIN			9966
#define WBO2_BITMAP_MAX			9999

typedef struct _graph_element
{
	UInt8	display, graph, log;
	double	ppr, range;
	double	low, high;
} graph_element;

#define ELEMENT_DISPLAY			0x01
#define ELEMENT_DISPLAY_SLOW	0x02

extern MemHandle			bitmap_handle_cache[ WBO2_BITMAP_MAX - WBO2_BITMAP_MIN + 1 ];
extern BitmapPtr			bitmap_ptr_cache[ WBO2_BITMAP_MAX - WBO2_BITMAP_MIN + 1 ];

#define PF_TEv1		0
#define PF_TEv15	1
#define PF_TEv20	2

extern UInt16				portID;
extern unsigned char		packet_type;
extern int					synced, packet_pos, packet_length, packet_crc_pos;
extern UInt32				Vwb, Vuser1, Vuser2, rpm, RPM;
extern UInt32				last_Vuser1, last_Vuser2, last_RPM;
extern int					lambda;
extern double				AFR, last_AFR;
extern unsigned int			sampleNo;
extern int					haveData, haveMarker, logActive;
extern int					drawDisplay;

extern int					graph_clear;

extern graph_element		graph_afr, graph_rpm, graph_v1, graph_v2;
extern UInt32				rpm_factor;
extern int					rpm_pulses;

/*
 * Database Structures/Variables
 */

#define WBO2_FIELD_VWB		0x0001
#define WBO2_FIELD_RPM		0x0002
#define WBO2_FIELD_VU1		0x0004
#define WBO2_FIELD_VU2		0x0008

#define WBO2_RECORD_HEADER_SIZE		12
#define WBO2_RECORD_MAX_SIZE		32768

typedef struct _WBO2Record_Header
{
	UInt16	next, prev;
	UInt32	samples;
	UInt32	data_offset;

	/*
	 * The following fields aren't written to the database,
	 * but are used during log replay
	 */

	UInt16						dbrecord_index;
	UInt32						first_sample;
	struct _WBO2Record_Header	*nextp, *prevp;
} WBO2Record_Header;

#define WBO2_RECORD_DESC_SIZE		1049

typedef struct _WBO2Record_Desc
{
	UInt16	fields;
	char	name[ 100 + 1 ];
	char	notes[ 923 + 1 ];
	char	v1_label[ 10 + 1 ];
	char	v2_label[ 10 + 1 ];
} WBO2Record_Desc;

extern DmOpenRef				dbP;
extern WBO2Record_Header		dbrecord_header;
extern WBO2Record_Desc			dbrecord_desc;
extern unsigned char			dbrecord_queue[ 1024 ];
extern int						dbrecord_queue_pos, dbrecord_queue_samples;
extern UInt16					dbrecord_index, dbrecord_first_index;
extern UInt32					dbrecord_size;
extern UInt32					dbrecord_item_size;

/*
 * Configuration Functions (config.c)
 */

void Config_Load() __attribute__ ((section("config")));
void Config_Save() __attribute__ ((section("config")));

/*
 * Logging Functions (log.c)
 */

void Log_Start() __attribute__ ((section("log")));
UInt32 Log_Read_Header( Char *recordp, UInt32 offset, WBO2Record_Header *header ) __attribute__ ((section("log")));
UInt32 Log_Write_Header( MemPtr recordp, UInt32 offset, WBO2Record_Header *header ) __attribute__ ((section("log")));
UInt32 Log_Read_Desc( Char *recordp, UInt32 offset, WBO2Record_Desc *desc ) __attribute__ ((section("log")));
UInt32 Log_Write_Desc( MemPtr recordp, UInt32 offset, WBO2Record_Desc *desc ) __attribute__ ((section("log")));
void Log_Flush() __attribute__ ((section("log")));
void Log_Data( int marker ) __attribute__ ((section("log")));
void Log_End() __attribute__ ((section("log")));

void Log_Delete_Chain( UInt16 dbrecord_index ) __attribute__ ((section("log")));
void Log_Read_Chain( UInt16 dbrecord_index, WBO2Record_Header **header_chain, WBO2Record_Desc *desc, UInt32 *total_samples ) __attribute__ ((section("log")));
void Log_Free_Chain( WBO2Record_Header **header_chain ) __attribute__ ((section("log")));

/*
 * AFR Related Functions (fmdlog.c)
 */

typedef struct _AFRLookup
{
	int		Vout;
	double	AFR;
} AFRLookup;

double AFR_Lookup( UInt32 v, AFRLookup *table, int n );

/*
 * Main Form Functions (fmdlog.c)
 */

#define LCD_X				0
#define LCD_Y				19

#define RPM_X				97
#define RPM_Y				19

#define U1L_X				126
#define U1L_Y				42

#define U1_X				128
#define U1_Y				43

#define U2L_X				126
#define U2L_Y				55

#define U2_X				128
#define U2_Y				56

#define GRAPH_SAMPLE_WIDTH	2
#define GRAPH_SAMPLES		79

#define GRAPH_MIN_X			1
#define GRAPH_MAX_X			158
#define GRAPH_WIDTH			158

#define GRAPH_MIN_Y			70
#define GRAPH_MAX_Y			158
#define GRAPH_HEIGHT		89

Boolean	MainForm_HandleEvent( EventPtr event );

void	Draw_AFR( double afr, int x, int y );
void	Erase_AFR( int x, int y );
void	Draw_RPM( unsigned int V, int x, int y );
void	Erase_RPM( int x, int y );
void	Draw_V( unsigned int V, int x, int y );
void	Erase_V( int x, int y );
void	Draw_Vlabel( const char *label, int x, int y );
void	Erase_Vlabel( const char *label, int x, int y );
void	FMDLog_DrawBitmap( int id, int x, int y );

void	Calculate_PPR();
void	Graph_Calculate_PPR();
int		MainForm_Graph_CalculateY( double data, graph_element *graph );
void	MainForm_DrawGraph();
void	MainForm_DrawMarker();

void	LoggedDataForm_LoadRecords( FormPtr frm );

void	DisableSerialHandler();
void	EnableSerialHandler();

/*
 * Save Log Form Functions (savelog.c)
 */

Boolean SaveLogForm_HandleEvent( EventPtr event ) __attribute__ ((section("savelog")));

/*
 * Edit Log Form Functions (editlog.c)
 */

Boolean EditLogForm_HandleEvent( EventPtr event ) __attribute__ ((section("editlog")));

/*
 * Hardware Form Functions (hardware.c)
 */

Boolean HardwareForm_HandleEvent( EventPtr event ) __attribute__ ((section("hw")));

/*
 * Preferences Form Functions (prefs.c)
 */

void Preferences_Set_Checkbox( FormPtr frm, UInt16 id, UInt8 value ) __attribute__ ((section("prefs")));
void Preferences_Set_AFR( FormPtr frm, UInt16 id, double value ) __attribute__ ((section("prefs")));
void Preferences_Set_Number( FormPtr frm, UInt16 id, UInt32 value ) __attribute__ ((section("prefs")));
void Preferences_Get_Checkbox( FormPtr frm, UInt16 id, UInt8 *value ) __attribute__ ((section("prefs")));
void Preferences_Get_AFR( FormPtr frm, UInt16 id, double *value ) __attribute__ ((section("prefs")));
void Preferences_Get_Number( FormPtr frm, UInt16 id, double *value ) __attribute__ ((section("prefs")));
void Preferences_Get_NumberU32( FormPtr frm, UInt16 id, UInt32 *value ) __attribute__ ((section("prefs")));
Boolean PreferencesForm_HandleEvent( EventPtr event ) __attribute__ ((section("prefs")));

/*
 * Log Replay Form Functions (logreplay.c)
 */

#define REPLAY_LCD_X	0
#define REPLAY_LCD_Y	19

#define REPLAY_RPM_X	97
#define REPLAY_RPM_Y	19

#define REPLAY_U1L_X	126
#define REPLAY_U1L_Y	42

#define REPLAY_U1_X		128
#define REPLAY_U1_Y		43

#define REPLAY_U2L_X	126
#define REPLAY_U2L_Y	55

#define REPLAY_U2_X		128
#define REPLAY_U2_Y		56

extern BitmapPtr			replaybitmap;
extern WinHandle			replaybitmap_window;
extern MemHandle			replayrecordH;
extern UInt16				replayrecord_dbindex;
extern UInt32				replayrecord_samples, replayrecord_visible_samples;
extern WBO2Record_Header	*replayrecord_chain, *replayrecord_current;
extern WBO2Record_Desc		replayrecord_desc;
extern unsigned char		*replaydatap;
extern int					replay_item_size;
extern int					replaycurrent;
extern int					replayleftoffset;
extern int					replaypush;
extern int					replaypushamount;
extern int					replay_lastleftoffset;
extern int					replay_lastcurrent;
extern Char					*replay_title;

typedef struct _LogReplayGraph_State
{
	UInt16 from_RPM, to_RPM,
		   from_Vuser1, to_Vuser1,
		   from_Vuser2, to_Vuser2;
	double from_AFR, to_AFR;
} LogReplayGraph_State;

int LogReplayForm_Graph_CalculateY( double data, graph_element *graph ) __attribute__ ((section("logrep")));
void LogReplayForm_Load( WBO2Record_Header *header ) __attribute__ ((section("logrep")));
void LogReplayForm_Data( int sample, double *rAFR, UInt16 *rRPM, UInt16 *rVuser1, UInt16 *rVuser2 ) __attribute__ ((section("logrep")));
void LogReplayForm_Record( int sample, double *rAFR, UInt16 *rRPM, UInt16 *rVuser1, UInt16 *rVuser2 ) __attribute__ ((section("logrep")));
void LogReplayForm_DrawGraphElement( int i, UInt16 xofs, LogReplayGraph_State *state ) __attribute__ ((section("logrep")));
void LogReplayForm_Draw( int full ) __attribute__ ((section("logrep")));
Boolean LogReplayForm_HandleEvent( EventPtr event ) __attribute__ ((section("logrep")));

/*
 * Scaling Functions (scaling.c)
 */

#define SCALING_STEPS		16

extern char *scaling_title;
extern UInt32 *scaling_mv, scaling_v1_mv[ SCALING_STEPS ], scaling_v2_mv[ SCALING_STEPS ];
extern UInt32 *scaling_out, scaling_v1_out[ SCALING_STEPS ], scaling_v2_out[ SCALING_STEPS ];
extern int *scaling_points, scaling_v1_points, scaling_v2_points;

UInt32 Sensor_Scale( UInt32 input, UInt32 *mv, UInt32 *out, int scaling_points ) __attribute__ ((section("scaling")));
Boolean ScalingForm_HandleEvent( EventPtr event ) __attribute__ ((section("scaling")));

/*
 * Smoothing Functions (smoothing.c)
 */

extern UInt32 smoothing_afr, smoothing_rpm, smoothing_v1, smoothing_v2, max_smoothing, skip_factor;

Boolean SmoothingForm_HandleEvent( EventPtr event ) __attribute__ ((section("smooth")));

/*
 * Label Dialog (labels.c)
 */

extern char v1_label[ 10 + 1 ], v2_label[ 10 + 1 ];

Boolean LabelsForm_HandleEvent( EventPtr event ) __attribute__ ((section("labels")));

/*
 * Alarm Functions (alarms.c)
 */

extern UInt32 alarm_min_RPM;
extern UInt8 alarm_afr, alarm_rpm, alarm_v1, alarm_v2;
extern double alarm_AFR_low, alarm_AFR_high;
extern UInt32 alarm_AFR_Hz,
			  alarm_RPM_low, alarm_RPM_high, alarm_RPM_Hz,
			  alarm_Vuser1_low, alarm_Vuser1_high, alarm_Vuser1_Hz,
			  alarm_Vuser2_low, alarm_Vuser2_high, alarm_Vuser2_Hz;

void Alarm_Play( int hz ) __attribute__ ((section("alarms")));
Boolean AlarmForm_HandleEvent( EventPtr event ) __attribute__ ((section("alarms")));

/*
 * About Functions (about.c)
 */

Boolean AboutForm_HandleEvent( EventPtr event );

/*
 * Color Dialog (colors.c)
 */

extern IndexedColorType	color_afr, color_rpm, color_v1, color_v2;

Boolean ColorForm_HandleEvent( EventPtr event ) __attribute__ ((section("colors")));

#endif

