/*
 * FMDLog
 * Wideband O2 Display/Datalogging Application
 * For the TechEdge DIY-WB series of devices
 * (c) 2002-2003, Jonathan Burchmore
 *
 * Your use of this software is governed by the GNU General Public License, v2.
 * See the file COPYING.txt, or http://www.fsf.org/copyleft/gpl.txt
 *
 * This software is provided with ABSOLUTELY NO WARRANTY.  See the GNU General
 * public license for additional information.
 */

#include <PalmOS.h>
#include "resource.h"
#include "fmdlog.h"

/*
 * Preferences Form Functions
 */

void Preferences_Set_Checkbox( FormPtr frm, UInt16 id, UInt8 value )
{
	CtlSetValue( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ), value );
}

void Preferences_Set_AFR( FormPtr frm, UInt16 id, double value )
{
	int i;
	Char *text;
	UInt8 data;
	MemHandle handle;

	handle	= MemHandleNew( 5 );
	text	= MemHandleLock( handle );

	data			= value * 10.0;
	i				= 0;

	if ( data / 100 )
	{
		text[ i++ ]	= '0' + data / 100;
	}

	text[ i++ ]		= '0' + ( ( data % 100 ) / 10 );
	text[ i++ ]		= '.';
	text[ i++ ]		= '0' + ( data % 10 );
	text[ i++ ]		= '\0';

	MemHandleUnlock( handle );
	FldSetTextHandle( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ), handle );
}

void Preferences_Set_Number( FormPtr frm, UInt16 id, UInt32 value )
{
	Char *text;
	MemHandle handle;

	handle	= MemHandleNew( 10 );
	text	= MemHandleLock( handle );

	StrPrintF( text, "%ld", value );

	MemHandleUnlock( handle );
	FldSetTextHandle( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ), handle );
}

void Preferences_Set_RPM( FormPtr frm, UInt16 id )
{

	if ( rpm_factor == 12000000 )	Preferences_Set_Number( frm, id, rpm_pulses );
	else							Preferences_Set_AFR( frm, id, 12000000.0 / rpm_factor );
}

void Preferences_Get_Checkbox( FormPtr frm, UInt16 id, UInt8 *value )
{
	*value = CtlGetValue( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ) );
}

void Preferences_Get_AFR( FormPtr frm, UInt16 id, double *value )
{
	Char *text;
	Char temp[ 10 ], *tempp;
	MemHandle handle;

	handle	= FldGetTextHandle( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ) );
	if ( handle == NULL )
	{
		*value	= 0.00;
	}
	else
	{
		text	= MemHandleLock( handle );

		StrCopy( temp, text );

		MemHandleUnlock( handle );

		if ( ( tempp = StrChr( temp, '.' ) ) != NULL )        *tempp++ = '\0';

		*value	= 0;

		switch ( StrLen( temp ) )
		{
			case 2 : *value += ( temp[ 0 ] - '0' ) * 10;
					 *value += ( temp[ 1 ] - '0' );
					 break;
			case 1 : *value += ( temp[ 0 ] - '0' );
					 break;
		}

		if ( tempp && *tempp )    *value += ( *tempp - '0' ) / 10.0;
	}
}

void Preferences_Get_Number( FormPtr frm, UInt16 id, double *value )
{
	Char *text;
	MemHandle handle;

	handle	= FldGetTextHandle( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ) );
	if ( handle == NULL )
	{
		*value = 0.0;
	}
	else
	{
		text	= MemHandleLock( handle );
		*value	= StrAToI( text );

		MemHandleUnlock( handle );
	}
}

void Preferences_Get_NumberU32( FormPtr frm, UInt16 id, UInt32 *value )
{
	Char *text;
	MemHandle handle;

	handle	= FldGetTextHandle( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ) );
	if ( handle == NULL )
	{
		*value = 0;
	}
	else
	{
		text	= MemHandleLock( handle );
		*value	= StrAToI( text );

		MemHandleUnlock( handle );
	}
}

void Preferences_Get_RPM( FormPtr frm, UInt16 id )
{
	Char *text;
	double value;
	Char temp[ 10 ], *tempp;
	MemHandle handle;

	handle	= FldGetTextHandle( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, id ) ) );
	if ( handle == NULL )
	{
		rpm_pulses	= 4;
		rpm_factor	= 12000000;
	}
	else
	{
		text	= MemHandleLock( handle );
		StrCopy( temp, text );
		MemHandleUnlock( handle );

		if ( ( tempp = StrChr( temp, '.' ) ) == NULL )
		{
			rpm_factor	= 12000000;
			rpm_pulses	= StrAToI( temp );
		}
		else
		{
			*tempp++	= '\0';
			value		= 0;

			switch ( StrLen( temp ) )
			{
				case 2 : value += ( temp[ 0 ] - '0' ) * 10;
						 value += ( temp[ 1 ] - '0' );
						 break;
				case 1 : value += ( temp[ 0 ] - '0' );
						 break;
			}

			if ( tempp && *tempp )    value += ( *tempp - '0' ) / 10.0;

			if ( value >= 1 )
			{
				rpm_factor	= 12000000;
				rpm_pulses	= ( int ) value;
			}
			else
			{
				rpm_factor	= 12000000 / value;
				rpm_pulses	= 1;
			}
		}
	}
}

Boolean PreferencesForm_HandleEvent( EventPtr event )
{
	FormPtr frm;
	UInt8 temp;

	if ( event->eType == frmOpenEvent )
	{
		frm				= FrmGetActiveForm();

		Preferences_Set_Checkbox( frm,	checkboxPrefsGraphAFR,	graph_afr.graph );
		Preferences_Set_Checkbox( frm,	checkboxPrefsGraphRPM,	graph_rpm.graph );
		Preferences_Set_Checkbox( frm,	checkboxPrefsGraphVu1,	graph_v1.graph );
		Preferences_Set_Checkbox( frm,	checkboxPrefsGraphVu2,	graph_v2.graph );

		Preferences_Set_AFR( frm,		textPrefsAFRLow,		graph_afr.low );
		Preferences_Set_AFR( frm,		textPrefsAFRHigh,		graph_afr.high );
		Preferences_Set_Number( frm,	textPrefsRPMLow,		graph_rpm.low );
		Preferences_Set_Number( frm,	textPrefsRPMHigh,		graph_rpm.high );
		Preferences_Set_Number( frm,	textPrefsVu1Low,		graph_v1.low );
		Preferences_Set_Number( frm,	textPrefsVu1High,		graph_v1.high );
		Preferences_Set_Number( frm,	textPrefsVu2Low,		graph_v2.low );
		Preferences_Set_Number( frm,	textPrefsVu2High,		graph_v2.high );

		Preferences_Set_Checkbox( frm,	checkboxPrefsLogAFR,	graph_afr.log );
		Preferences_Set_Checkbox( frm,	checkboxPrefsLogRPM,	graph_rpm.log );
		Preferences_Set_Checkbox( frm,	checkboxPrefsLogVu1,	graph_v1.log );
		Preferences_Set_Checkbox( frm,	checkboxPrefsLogVu2,	graph_v2.log );

		Preferences_Set_RPM( frm,		textPrefsRPMPulses );

		if ( lambda )	CtlSetLabel( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, popupMixtureMode ) ), "Lambda" );
		else			CtlSetLabel( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, popupMixtureMode ) ), "A/F (Gas)" );

		LstSetSelection( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, listMixtureModes ) ), lambda );

		Preferences_Set_Checkbox( frm,	checkboxSlowAFR,		graph_afr.display & ELEMENT_DISPLAY_SLOW );

		FrmDrawForm( frm );
		return true;
	}
	else if ( event->eType == ctlSelectEvent )
	{
		if ( event->data.ctlSelect.controlID == buttonPrefsSave )
		{
			frm				= FrmGetActiveForm();

			Preferences_Get_Checkbox( frm,	checkboxPrefsGraphAFR,	&graph_afr.graph );
			Preferences_Get_Checkbox( frm,	checkboxPrefsGraphRPM,	&graph_rpm.graph );
			Preferences_Get_Checkbox( frm,	checkboxPrefsGraphVu1,	&graph_v1.graph );
			Preferences_Get_Checkbox( frm,	checkboxPrefsGraphVu2,	&graph_v2.graph );

			Preferences_Get_AFR( frm,		textPrefsAFRLow,		&graph_afr.low );
			Preferences_Get_AFR( frm,		textPrefsAFRHigh,		&graph_afr.high );
			Preferences_Get_Number( frm,	textPrefsRPMLow,		&graph_rpm.low );
			Preferences_Get_Number( frm,	textPrefsRPMHigh,		&graph_rpm.high );
			Preferences_Get_Number( frm,	textPrefsVu1Low,		&graph_v1.low );
			Preferences_Get_Number( frm,	textPrefsVu1High,		&graph_v1.high );
			Preferences_Get_Number( frm,	textPrefsVu2Low,		&graph_v2.low );
			Preferences_Get_Number( frm,	textPrefsVu2High,		&graph_v2.high );

			Preferences_Get_Checkbox( frm,	checkboxPrefsLogAFR,	&graph_afr.log );
			Preferences_Get_Checkbox( frm,	checkboxPrefsLogRPM,	&graph_rpm.log );
			Preferences_Get_Checkbox( frm,	checkboxPrefsLogVu1,	&graph_v1.log );
			Preferences_Get_Checkbox( frm,	checkboxPrefsLogVu2,	&graph_v2.log );

			graph_afr.display	= graph_afr.graph;
			graph_rpm.display	= graph_rpm.graph;
			graph_v1.display	= graph_v1.graph;
			graph_v2.display	= graph_v2.graph;

			Preferences_Get_RPM( frm,		textPrefsRPMPulses );

			lambda				= LstGetSelection( FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, listMixtureModes ) ) );

			Preferences_Get_Checkbox( frm,	checkboxSlowAFR,		&temp );
			if ( temp )	graph_afr.display	|= ELEMENT_DISPLAY_SLOW;

			Config_Save();
			Graph_Calculate_PPR();

			FrmReturnToForm( 0 );

			if ( graph_afr.display )	Draw_AFR( AFR, LCD_X, LCD_Y );
			else						Erase_AFR( LCD_X, LCD_Y );

			if ( graph_rpm.display )	Draw_RPM( RPM,	RPM_X,	RPM_Y );
			else						Erase_RPM( RPM_X, RPM_Y );

			if ( graph_v1.display )
			{
				Draw_V( Vuser1,	U1_X,	U1_Y );
				Draw_Vlabel( v1_label,	U1L_X,	U1L_Y );
			}
			else
			{
				Erase_V( U1_X, U1_Y );
				Erase_Vlabel( v1_label,	U1L_X,	U1L_Y );
			}

			if ( graph_v2.display )	
			{
				Draw_V( Vuser2,	U2_X,	U2_Y );
				Draw_Vlabel( v2_label,	U2L_X,	U2L_Y );
			}
			else
			{
				Erase_V( U2_X, U2_Y );
				Erase_Vlabel( v2_label,	U2L_X,	U2L_Y );
			}

			return true;
		}
		else if ( event->data.ctlSelect.controlID == buttonPrefsCancel )
		{
			FrmReturnToForm( 0 );
			return true;
		}
	}

	return false;
}

