#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

#define SCALE( x )			( ( x * 5000 ) / 8192 )
#define RPM_PULSES_PER_REV	4

int Read( int fd, unsigned char *buffer, int size )
{
	int numread, totalread;

	for ( totalread = 0; totalread < size; totalread += numread )
	{
		if ( ( numread = read( fd, &buffer[ totalread ], size - totalread ) ) < 0 )	return -1;
	}

	return size;
}

void main()
{
	int synced, fd;
	unsigned int Vwb, Vuser1, Vuser2, rpm, RPM;
	unsigned char packet[ 12 ], crc;

	synced	= 0;
	
	if ( ( fd = open( "/dev/ttyS0", O_RDONLY ) ) < 0 )	goto error;
	for ( ;; )
	{
		if ( synced )
		{
			if ( Read( fd, packet, 12 ) != 12 )			goto error;
		}
		else
		{
			printf( "." );	fflush( stdout );

			if ( read( fd, &packet[ 0 ], 1 ) != 1 )		goto error;
			if ( packet[ 0 ] != 0x5A )					continue;

			if ( read( fd, &packet[ 1 ], 1 ) != 1 )		goto error;
			if ( packet[ 1 ] != 0xA5 )					continue;

			synced = 1;
			printf( "Synchronized\n" );

			if ( Read( fd, &packet[ 2 ], 10 ) != 10 )	goto error;
		}

		crc	= ~( packet[ 0 ] + packet[ 1 ] + packet[ 2 ] + packet[ 3 ] +
				 packet[ 4 ] + packet[ 5 ] + packet[ 6 ] + packet[ 7 ] +
				 packet[ 8 ] + packet[ 9 ] + packet[ 10 ] );

		if ( crc != packet[ 11 ] )
		{
			printf( "CRC Mismatch\n" );
			synced = 0;
		}
		else
		{
			Vwb		= ( ( packet[ 3 ] & 0x1F ) << 8 ) | packet[ 4 ];
			Vuser1	= ( ( packet[ 5 ] & 0x1F ) << 8 ) | packet[ 6 ]; 
			Vuser2	= ( ( packet[ 7 ] & 0x1F ) << 8 ) | packet[ 8 ];
			rpm		= ( ( packet[ 9 ] & 0x1F ) << 8 ) | packet[ 10 ];

			if ( rpm == 0 )
			{
				RPM	= 0;
			}
			else
			{
				RPM	= ( ( 1000000.0 / 5.0 / rpm ) * 60 ) / RPM_PULSES_PER_REV;
				RPM	= 12000000 / RPM_PULSES_PER_REV / rpm;
			}

			printf( "Packet\n" );
			printf( "\tCounter: %d\n", packet[ 2 ] );
			printf( "\tVwb    : %d %d %d\n", SCALE( Vwb ), ( unsigned char ) ( SCALE( Vwb ) / 10 ), Vwb );
			printf( "\tAFR    : %.2f\n", SCALE( Vwb ) / 100.0 );
			printf( "\tVuser1 : %d\n", SCALE( Vuser1 ) );
			printf( "\tVuser2 : %d\n", SCALE( Vuser2 ) );
			printf( "\trpm    : %d\n", rpm );
			printf( "\tRPM    : %d\n", RPM );
		}
	}

	return;

error:
	printf( "Error\n" );
}
